package com.gree.mall.manager.logic.common;

import cn.binarywang.wx.miniapp.bean.scheme.WxMaGenerateSchemeRequest;
import com.alibaba.fastjson.JSON;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.admin.AdminWebsitBean;
import com.gree.mall.manager.commonmapper.AdminMapper;
import com.gree.mall.manager.config.wx.WxConfiguration;
import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.*;
import com.gree.mall.manager.utils.ApplicationContextUtils;
import com.gree.mall.manager.utils.CommonUtils;
import com.gree.mall.manager.utils.RedisUtil;
import com.gree.mall.manager.utils.oss.OSSUtil;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.lang.time.DateUtils;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CommonLogic {

    @Value("${spring.profiles.active}")
    private String active;
    @Value("${wechat.payment.notifyUrl}")
    private String notifyUrl;
    //    @Value("${wechat.payment.refundNotifyUrl}")
//    private String refundNotifyUrl;
    @Value("${wechat.keyPath}")
    private String keyPath;

    @Autowired
    WechatLogic wechatLogic;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    AdminCompanyWechatService AdminCompanyWechatService;
    @Autowired
    CommonFileService commonFileService;
    @Autowired
    OrderInfoService orderInfoService;
    @Autowired
    RegionService regionService;
    @Autowired
    AdminUserService adminUserService;
    @Autowired
    AdminUserCompanyRelaService adminUserCompanyRelaService;
    @Autowired
    AdminCompanyWechatService adminCompanyWechatService;
    @Autowired
    AdminUserWebsitRelaService adminUserWebsitRelaService;
    @Autowired
    AdminMapper adminMapper;


    public AdminUserCom getAdminUser(){
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return this.getAdminUser(request);
    }

    /**
     * 获取帐号以及部门id集合
     */
    public AdminUserCom getAdminUser(HttpServletRequest request){
        String userId = CommonUtils.getUserId(request);
        //方便测试用
        if(userId == null && !active.equals("prd")){
            AdminUser admin = adminUserService.lambdaQuery().eq(AdminUser::getUserName, "admin").one();
            AdminUserCom adminUserCom = new AdminUserCom();
            BeanUtils.copyProperties(admin,adminUserCom);
            AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById("1");
            adminUserCom.setAdminCompanyWechat(adminCompanyWechat);
            adminUserCom.setLoginCompanyWechatId(adminCompanyWechat.getCompanyWechatId());
            adminUserCom.setLoginCompanyName(adminCompanyWechat.getCompanyName());
            return adminUserCom;
        }
        AdminUser adminUser = adminUserService.getById(userId);
        if(adminUser == null || !adminUser.getStatus()){
            throw new RemoteServiceException("帐号不存在或已被禁用");
        }
        AdminUserCom adminUserCom = new AdminUserCom();
        BeanUtils.copyProperties(adminUser,adminUserCom);

        //企业负责人/超管
        AdminUserCom account = this.companyAccount(adminUserCom);
        if(account != null){
            return account;
        }
        //部门帐号
        AdminUserCom account2 = this.websitAccount(adminUserCom,request);
        if(account2 != null){
            return account2;
        }
        //不过滤权限
        adminUserCom.setAdminWebsitIds(null);
        return adminUserCom;
    }

    /**
     * 超管用户，企业主帐号
     */
    public AdminUserCom companyAccount(AdminUserCom adminUser){
        if(adminUser.getType() != 0){
            //非超管
            if(!adminUser.getUserName().equals("admin")) {
                String adminUserId = adminUser.getAdminUserId();
                //查询该用户拥有的部门权限
                //List<AdminUserCompanyRela> list = adminUserCompanyRelaService.lambdaQuery().eq(AdminUserCompanyRela::getAdminUserId, adminUser.getAdminUserId()).list();
                List<AdminCompanyWechat> list = adminMapper.queryAdminCompanyWechat(adminUserId);
                if (CollectionUtils.isEmpty(list)) {
                    throw new RemoteServiceException("暂无授权，请联系管理员");
                }
                List<String> companyWechatIds = list.stream().map(AdminCompanyWechat::getCompanyWechatId).distinct().collect(Collectors.toList());
                List<String> corpIds = list.stream().map(AdminCompanyWechat::getCorpId).collect(Collectors.toList());

                adminUser.setCompanyWechatIds(companyWechatIds);
                adminUser.setCorpIds(corpIds);
                adminUser.setCorpId(corpIds.get(0));
                adminUser.setOnlyRead(companyWechatIds.size() > 1);
                if (companyWechatIds.size() == 1) {
                    AdminCompanyWechat adminCompanyWechat = list.get(0);
                    adminUser.setLoginCompanyWechatId(adminCompanyWechat.getCompanyWechatId());
                    adminUser.setLoginCompanyName(adminCompanyWechat.getCompanyName());
                    adminUser.setAdminCompanyWechat(adminCompanyWechat);
                }
                adminUser.setCompanyWechatId(companyWechatIds.get(0));
            }
            return adminUser;
        }
        return null;
    }


    /**
     * 部门帐号
     */
    public AdminUserCom websitAccount(AdminUserCom adminUserCom,HttpServletRequest request){
        String userId = adminUserCom.getAdminUserId();
        List<AdminUserWebsitRela> websitList = adminUserWebsitRelaService.lambdaQuery().eq(AdminUserWebsitRela::getAdminUserId, userId).list();
        if(CollectionUtils.isEmpty(websitList)){
            throw new RemoteServiceException("暂无授权,请联系管理员");
        }
        List<String> adminWebsitIds = websitList.stream().map(AdminUserWebsitRela::getAdminWebsitId).distinct().collect(Collectors.toList());
        List<String> companyWechatIds = websitList.stream().map(AdminUserWebsitRela::getCompanyWechatId).distinct().collect(Collectors.toList());
        adminUserCom.setCompanyWechatIds(companyWechatIds);
        adminUserCom.setAdminWebsitIds(adminWebsitIds);
        adminUserCom.setOnlyRead(companyWechatIds.size() > 1);

        String loginCompanyWechatId = "";
        String loginCompanyName = "";
        if(companyWechatIds.size() == 1){
            loginCompanyWechatId = websitList.get(0).getCompanyWechatId();
            AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById(loginCompanyWechatId);
            loginCompanyName = adminCompanyWechat.getCompanyName();
        }
        //获取当前登录人选用的企业微信身份

        if(StringUtils.isEmpty(loginCompanyWechatId)){
            loginCompanyWechatId = request.getHeader("loginCompanyWechatId");
        }
        if(StringUtils.isEmpty(loginCompanyWechatId)){
            loginCompanyWechatId = request.getParameter("loginCompanyWechatId");
        }
        if(StringUtils.isEmpty(loginCompanyName)){
            loginCompanyName = request.getHeader("loginCompanyName");
        }
        if(StringUtils.isEmpty(loginCompanyName)){
            loginCompanyName = request.getParameter("loginCompanyName");
        }

        if(StringUtils.isNotEmpty(loginCompanyWechatId)) {

            if (!adminUserCom.getCompanyWechatIds().contains(loginCompanyWechatId)) {
                throw new RemoteServiceException("暂无权限");
            }
            adminUserCom.setLoginCompanyWechatId(loginCompanyWechatId);
            adminUserCom.setLoginCompanyName(loginCompanyName);
            //获取当前登录人的企业微信配置
            AdminCompanyWechat adminCompanyWechat = this.getAdminCompanyWechat(loginCompanyWechatId);
            adminUserCom.setAdminCompanyWechat(adminCompanyWechat);
            adminUserCom.setCorpIds(Arrays.asList(adminCompanyWechat.getCorpId()));
            adminUserCom.setCorpId(adminCompanyWechat.getCorpId());
        }
        return adminUserCom;
    }

    /**
     * 获取企业微信配置
     */
    public AdminCompanyWechat getAdminCompanyWechat(String companyWechatId){
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById(companyWechatId);
        return adminCompanyWechat;
    }
    /**
     * 获取所有企业微信配置
     */
    public List<AdminCompanyWechat> getAdminCompanyList(){
        List<AdminCompanyWechat> list = adminCompanyWechatService.list();
        return list;
    }

    /**
     * 刷新微信相关配置
     */
    public void reloadWxConfig(){
        List<AdminCompanyWechat> list = AdminCompanyWechatService.list();
        log.info("【刷新微信配置开始】");
        WxConfiguration.reloadWxConfig(list,redisUtil,keyPath,notifyUrl);
        log.info("【刷新微信配置成功】");
    }

    /**
     * 附件归属绑定
     */
    public void bindFile(String objId,String objType,List<String> fileIds){
        //先删除后绑定
        commonFileService.lambdaUpdate()
                .eq(CommonFile::getObjId,objId)
                .eq(CommonFile::getObjType,objType)
                .notIn(CommonFile::getId,fileIds).remove();

        for(int i =0;i< fileIds.size();i++) {
            //这里分开每次存储的时候，每个图片创建时间加1秒，方便图片排序用
            commonFileService.lambdaUpdate()
                    .set(CommonFile::getObjId, objId)
                    .set(CommonFile::getObjType, objType)
                    .set(CommonFile::getCreateTime,DateUtils.addSeconds(new Date(),i))
                    .eq(CommonFile::getId, fileIds.get(i))
                    .update();
        }
    }

    /**
     * 查询附件列表
     */
    public List<CommonFile> queryFileByObjId(String objId,String objType){
        return commonFileService.lambdaQuery()
                .eq(CommonFile::getObjId,objId)
                .eq(CommonFile::getObjType,objType)
                .orderByAsc(CommonFile::getCreateTime)
                .list();
    }

    /**
     * 查询附件列表
     */
    public List<String> queryFileUrlsByObjId(String objId,String objType){
        List<CommonFile> commonFiles = this.queryFileByObjId(objId, objType);
        if(commonFiles.size() == 0){
            return null;
        }
        return commonFiles.stream().map(CommonFile::getUrl).collect(Collectors.toList());
    }


    /**
     * 上传附件
     * @param file
     * @return
     * @throws IOException
     */
    public CommonFile uploadFile( MultipartFile file) throws IOException, RemoteServiceException {

        if (file == null || file.isEmpty()) {
            return null;
        }
        String uploadFilePath = file.getOriginalFilename().replaceAll("\\\\", "/");
        String uploadFileName = uploadFilePath.substring(
                uploadFilePath.lastIndexOf('/') + 1, uploadFilePath.lastIndexOf('.'));
        String uploadFileSuffix = uploadFilePath.substring(uploadFilePath.lastIndexOf('.') + 1, uploadFilePath.length());
        long currentTime = System.currentTimeMillis();
        BufferedOutputStream bufferedOutputStream = null;

        //限制大小为10m
        if (file.getSize() > 1024 * 1024 * 30) {
            throw new RemoteServiceException("文件大小超出限制");
        }

        try {
            String filePath = OSSUtil.OSS_TYPE_PIC + currentTime + UUID.randomUUID().toString() + "." + uploadFileSuffix;
            if (!OSSUtil.uploadFile(filePath, file.getBytes())) {
                throw new RemoteServiceException("文件同步oss失败");
            }
            String ossUrl = OSSUtil.accessUrl + filePath;

            CommonFile commonFile = new CommonFile();
            commonFile.setUrl(ossUrl);
            commonFile.setName(uploadFileName);
            commonFile.setFileType(uploadFileSuffix);
            commonFile.setFileSize(BigDecimal.valueOf(file.getSize()));
            commonFile.setCreateTime(new Date());
            commonFileService.save(commonFile);
            return commonFile;
        } finally {
            if (bufferedOutputStream != null) {
                bufferedOutputStream.close();
            }
        }
    }


    public List<Region> queryRegionList(Long pid){
        return regionService.lambdaQuery().eq(Region::getPid,pid).list();
    }

    public String getFile(String key, HttpServletResponse response) throws IOException {
        String url = OSSUtil.getUrlWw(key);
        return url;
    }

    public String getUrlScheme(String query,String companyWechatId) throws IOException, WxErrorException {
        if(StringUtils.isEmpty(companyWechatId)){
            companyWechatId = Constant.GD_COMPANY_WECHAT_ID;
        }
        String accessToken = wechatLogic.getMaService(companyWechatId).getAccessToken();
        HttpPost httpPost = new HttpPost("https://api.weixin.qq.com/wxa/generatescheme?access_token="+accessToken);
        httpPost.setHeader("Content-Type","application/json; charset=utf-8");
        httpPost.setConfig(RequestConfig.custom().setConnectTimeout(10*1000).build());

        TreeMap<String,String>  jump_wxa=new TreeMap<>();
        jump_wxa.put("query",query);

        TreeMap<String,Object> treeMap=new TreeMap<>();
        treeMap.put("jump_wxa",jump_wxa);
        treeMap.put("is_expire", true);
        treeMap.put("expire_type", 1);
        treeMap.put("expire_interval", 7);
        httpPost.setEntity(new StringEntity(JSON.toJSONString(treeMap)));

        HttpResponse httpResponse = HttpClientBuilder.create().build().execute(httpPost);
        String s = EntityUtils.toString(httpResponse.getEntity());
        if(httpResponse.getStatusLine().getStatusCode() != HttpStatus.SC_OK){
            log.info("调用微信接口失败:"+httpPost.getURI());
            log.info("返回数据："+s);
            throw new RuntimeException("调用微信接口失败");
        }
        log.info("调用微信接口成功:"+httpPost.getURI());
        log.info("微信接口返回数据："+s);

        try{
            return JSON.parseObject(s).get("openlink").toString();
        }catch (NullPointerException e){
            return s;
        }
    }

    public String getFileGet(String key) {
        String url = OSSUtil.getUrlWw(key,null);
        return url;
    }
}
