package com.gree.mall.manager.schedule;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.gree.mall.manager.bean.delay.Job;
import com.gree.mall.manager.bean.sop.*;
import com.gree.mall.manager.bean.sop.File;
import com.gree.mall.manager.commonmapper.SopRulePushMapper;
import com.gree.mall.manager.config.wx.WxConfiguration;
import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.constant.DelayConfig;
import com.gree.mall.manager.constant.JobStatus;
import com.gree.mall.manager.enums.JobTopic;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.logic.common.WorkWechatLogic;
import com.gree.mall.manager.logic.sop.JobLogic;
import com.gree.mall.manager.logic.wxCustomer.SingleChatMessageLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.entity.MarketingMaterials;
import com.gree.mall.manager.plus.entity.SopPushTask;
import com.gree.mall.manager.plus.service.*;
import com.gree.mall.manager.utils.CommonUtils;
import com.gree.mall.manager.utils.DateUtils;
import com.gree.mall.manager.utils.WXPushUtils;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.cp.api.WxCpService;
import me.chanjar.weixin.cp.bean.article.MpnewsArticle;
import me.chanjar.weixin.cp.bean.article.NewArticle;
import me.chanjar.weixin.cp.bean.external.contact.FollowedUser;
import me.chanjar.weixin.cp.bean.external.contact.WxCpExternalContactInfo;
import me.chanjar.weixin.cp.bean.message.WxCpMessage;
import org.omg.CORBA.OBJ_ADAPTER;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.*;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

/**
 * 推送redis延迟消息定时任务
 */
@Slf4j
@Component
public class CarryJobScheduled {

    @Autowired
    private JobLogic jobLogic;

    @Autowired
    private SopPushTaskService sopPushTaskService;

    @Autowired
    private CommonLogic commonLogic;

    @Autowired
    private RedisLockRegistry redisLockRegistry;

    @Autowired
    private SingleChatMessageLogic singleChatMessageLogic;

    @Autowired
    private MarketingMaterialsService marketingMaterialsService;

    @Autowired
    private SopWorkerPushService sopWorkerPushService;


    @Autowired
    private AdminCompanyWechatService adminCompanyWechatService;

    static {
    /*    if (isWindows()) {
            System.loadLibrary("WeWorkFinanceSdk");

        }*/
        if (!isWindows()) {
            java.io.File file1 = new java.io.File("/tmp/file/libWeWorkFinanceSdk_Java.so");
            log.info("加载文件开始");
            if (file1.exists()) {
                System.load("/tmp/file/libWeWorkFinanceSdk_Java.so");
            } else {
                try {

                    InputStream stream = ResourceUtil.getStream("static/libWeWorkFinanceSdk_Java.so");
                    cn.hutool.core.io.file.FileWriter writer = new cn.hutool.core.io.file.FileWriter("/tmp/file/libWeWorkFinanceSdk_Java.so");
                    writer.writeFromStream(stream);
                    log.info("写入文件结束");
                } catch (Exception e) {
                    log.error(e.getMessage());
                }
                System.load("/tmp/file/libWeWorkFinanceSdk_Java.so");
                log.info("加载成功");
            }
        }
    }


    public static boolean isWindows() {
        String osName = System.getProperties().getProperty("os.name");
        return osName.toUpperCase().contains("WIN");
    }

    /**
     * 启动定时开启搬运JOB信息
     */
    //@Scheduled(cron = "*/1 * * * * *")
    @Scheduled(fixedDelay = 10000)
    public void carryJobToQueue() {
        Job processJob = jobLogic.getProcessJob(JobTopic.TOPIC_SOP.getTopic());
        if (processJob == null || processJob.getId() == null)
            return;
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_SOP_SYNC + processJob.getId());
        try {
            if (!obtain.tryLock(5, TimeUnit.SECONDS)) {
                throw new RemoteServiceException("系统繁忙，请稍后再试");
            }
            Map<String, String> map = (Map) JSONUtil.parseObj(processJob.getMessage());
            jobLogic.finishJob(processJob.getId());
            SopPushTask sopPushTask = sopPushTaskService.getById(map.get("id"));
            MarketingMaterials marketingMaterials = marketingMaterialsService.getById(sopPushTask.getMarketingMaterialsId());
            WxCpMessage wxCpMessage = new WxCpMessage();
            wxCpMessage.setToUser(map.get("toUser"));
            SopWorkerPush sopWorkerPush = new SopWorkerPush();
            sopWorkerPush.setId(map.get("pushWorkerId"));
            sopWorkerPush.setWorkerStatus(1);
            sopWorkerPush.setTaskTime(new Date());
            sopWorkerPush.setWorkerTime(new Date());
            sopWorkerPushService.updateById(sopWorkerPush);

            Map<String, Object> mapTO = new HashMap<>();
            //发送内容
            if (sopPushTask.getSendType().equals(1)) {
                wxCpMessage.setMsgType("text");
                wxCpMessage.setContent(sopPushTask.getSendText());
                Text text = new Text();
                text.setContent(sopPushTask.getSendText());
                mapTO.put("text", text);
            }
            if (sopPushTask.getSendType().equals(2)) {
                wxCpMessage.setMsgType("image");
                wxCpMessage.setMediaId(marketingMaterials.getMediaId());
                List<SopPushAdd> sopPushAd = new ArrayList<>();
                SopPushAdd sopPushAdd = new SopPushAdd();
                sopPushAdd.setMsgtype("image");
                Image image = new Image();
                image.setMedia_id(marketingMaterials.getMediaId());
                sopPushAdd.setImage(image);
                sopPushAd.add(sopPushAdd);
                mapTO.put("attachments", sopPushAd);
            }
            if (sopPushTask.getSendType().equals(3)) {
                wxCpMessage.setMsgType("video");
                wxCpMessage.setMediaId(marketingMaterials.getMediaId());
                wxCpMessage.setTitle("视频消息");
                wxCpMessage.setDescription(sopPushTask.getSendText());

                List<SopPushAdd> sopPushAd = new ArrayList<>();
                SopPushAdd sopPushAdd = new SopPushAdd();
                sopPushAdd.setMsgtype("video");
                Video video = new Video();
                video.setMedia_id(marketingMaterials.getMediaId());
                sopPushAdd.setVideo(video);
                sopPushAd.add(sopPushAdd);
                mapTO.put("attachments", sopPushAd);
            }

            if (sopPushTask.getSendType().equals(4)) {
                wxCpMessage.setMsgType("mpnews");
                List<MpnewsArticle> articles = new ArrayList<>();
                MpnewsArticle newArticle = new MpnewsArticle();
                newArticle.setTitle("网页消息：" + sopPushTask.getSendUrlName());
                newArticle.setThumbMediaId(marketingMaterials.getMediaId());
                newArticle.setContent(sopPushTask.getSendText() + "，<a href=\"" + marketingMaterials.getUrl() + "\">点击网页</a>");
                articles.add(newArticle);
                wxCpMessage.setMpnewsArticles(articles);

                List<SopPushAdd> sopPushAd = new ArrayList<>();
                SopPushAdd sopPushAdd = new SopPushAdd();
                sopPushAdd.setMsgtype("link");
                Link link = new Link();
                link.setTitle(sopPushTask.getSendUrlName());
                link.setUrl(marketingMaterials.getUrl());
                sopPushAdd.setLink(link);
                sopPushAd.add(sopPushAdd);
                mapTO.put("attachments", sopPushAd);

            }

            if (sopPushTask.getSendType().equals(5)) {
                wxCpMessage.setMsgType("text");
                wxCpMessage.setContent(sopPushTask.getSendText() + "小程序链接：" + commonLogic.getUrlScheme(null, map.get("companyWechatId")));

                Text text = new Text();
                text.setContent(sopPushTask.getSendText() + "小程序链接：" + commonLogic.getUrlScheme(null, map.get("companyWechatId")));
                mapTO.put("text", text);

            }

            if (sopPushTask.getSendType().equals(6)) {
                wxCpMessage.setMsgType("file");
                wxCpMessage.setMediaId(marketingMaterials.getMediaId());

                List<SopPushAdd> sopPushAd = new ArrayList<>();
                SopPushAdd sopPushAdd = new SopPushAdd();
                sopPushAdd.setMsgtype("file");
                File file = new File();
                file.setMedia_id(marketingMaterials.getMediaId());
                sopPushAdd.setFile(file);
                sopPushAd.add(sopPushAdd);
                mapTO.put("attachments", sopPushAd);
            }

            wxCpMessage.setAgentId(Convert.toInt(map.get("agentId1")));
            WxCpService wxCpService = WxConfiguration.cpServices.get(map.get("companyWechatId")).get(map.get("agentId1"));
            String accessToken = wxCpService.getAccessToken();
            push(map, accessToken, mapTO);
            WXPushUtils.sendToEnterpriseWechatWorker(wxCpService, wxCpMessage);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            obtain.unlock();
        }
    }

    public void push(Map<String, String> map, String accessToken, Map<String, Object> mapTO) {
        List<String> externalUserid = new ArrayList<>();
        externalUserid.add(map.get("externalUserID"));
        mapTO.put("external_userid", externalUserid);
        mapTO.put("sender", map.get("toUser"));
        mapTO.put("access_token", accessToken);
        JSONObject object = new JSONObject(mapTO);
        String post = HttpRequest.post(Constant.WXUrl.WX_PUSH_MSG_URL + accessToken)
                .body(object.toString())
                .execute().body();
        log.info("=====================" + post);
    }

    @Scheduled(fixedDelay = 1000 * 60 * 5)
    public synchronized void finance() throws InterruptedException {
        List<AdminCompanyWechat> list = adminCompanyWechatService.lambdaQuery().eq(AdminCompanyWechat::getTurnOn, 1).list();
        if (CollectionUtils.isEmpty(list))
            return;
        for (AdminCompanyWechat adminCompanyWechat : list) {
            singleChatMessageLogic.syncChatMessage(null, adminCompanyWechat.getSignSecret(), adminCompanyWechat.getPrivateKey(), adminCompanyWechat.getCorpId());
        }

    }


}
